/*-
 *
 * Brocade Proprietary and Confidential.
 * Copyright © 2018 Brocade Communications Systems LLC.
 * All Rights Reserved.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 */
import * as util from '../util/util.js';
import moment from 'moment';

const LOGGER = util.logger(__filename);

const asHighchartModel = (params)=>{
  const twoDimensionalData = params.data;   
  const columnVectorFromMatrixParams = {};
  columnVectorFromMatrixParams.data=twoDimensionalData;
  columnVectorFromMatrixParams.columnIndex = 0;
  const xAxisColumnWithHeader = columnVectorFromMatrix(columnVectorFromMatrixParams);
  const xAxisColumnWithoutHeader = xAxisColumnWithHeader.slice(1);
  let xAxis;
  LOGGER.info( ' params.config ' , JSON.stringify(params.config));
  const reportTimeZoneId = util.reportConfigProvider().reportTimeZoneId ;
  LOGGER.info(' time zone ', reportTimeZoneId);
  if (params.config.xAxisLabelDatetimeFormat){
    const formatXAxis = (date)=>{      
      if (reportTimeZoneId){
        return moment(date).tz(reportTimeZoneId).format(params.config.xAxisLabelDatetimeFormat);    
      }
      return moment(date).format(params.config.xAxisLabelDatetimeFormat);          
    }
    const xAxisColumnWithoutHeaderFormatted =  xAxisColumnWithoutHeader.map(formatXAxis);
    xAxis = { categories:xAxisColumnWithoutHeaderFormatted};      
  }else{
    xAxis = { categories:xAxisColumnWithoutHeader};      
  }
  let yAxis;
  if (params.config.measuresUomLocaleMessageId){
    const yAxisTitle = util.resolveMessage(params.config.measuresUomLocaleMessageId);
    yAxis = { 
              "title": {
                "enabled": true,
                "text": yAxisTitle            
              }              
            };      
  }  
  const series = [];
  const seriesYAxis = params.config.seriesYAxis;  
  const yAxisConfig = params.config.yAxis;
  LOGGER.info(" series y axis , yAxisConfig" , JSON.stringify(seriesYAxis), JSON.stringify(yAxisConfig));
  const colCount = twoDimensionalData[0].length;  
  for(let colIndex=1;colIndex<colCount;colIndex++){
    columnVectorFromMatrixParams.columnIndex = colIndex;
    const seriesColumn = columnVectorFromMatrix(columnVectorFromMatrixParams);
    const seriesParams = {data:seriesColumn};
    const seriesForColumn = columnVectorAsSeries(seriesParams);  
    if (seriesYAxis){
      const yAxisIndex = seriesYAxis[colIndex-1];
      if (yAxisIndex){
        seriesForColumn.yAxis=yAxisIndex;
      }
      LOGGER.info(" series for column ", JSON.stringify(seriesForColumn));
    }
    
    
    series.push(seriesForColumn);    
  }
  const hasSeries = twoDimensionalData.length > 1; 
  // flag to verify whether the chart has data in the series to plot;
  // if no valid data is found, we no need to covert the data as SVG.  
  const chart = {xAxis,yAxis,series,hasSeries};    
  let  finalChart;
  if (params.config.chartStyle.chart.type === 'pie'){ 
    // pie/donut chart is a special case     
    finalChart = mergePieChartDataAndStyle(chart,params.config.chartStyle);
  } else {
    if (yAxisConfig){
      const applyYAxisConfig = (axisConfig,index)=>{
        const title = util.resolveMessage(axisConfig.titleLocaleId);
        params.config.chartStyle.yAxis[index].title.text=title;
        params.config.chartStyle.yAxis[index].labels.format=axisConfig.labelFormat;
      }
      yAxisConfig.forEach(applyYAxisConfig);
      //LOGGER.info(" params.config.chartStyle ");
    }
    finalChart =  applyStyle(chart,params.config.chartStyle);
  }
  
 
  const twoSpaces = '  ';
  // LOGGER.info(JSON.stringify(finalChart,null,twoSpaces));
  return finalChart;
}

const applyStyle = (chart,chartStyle)=>{
  LOGGER.info("::: chart config ::: chart utils",JSON.stringify(chart));
  const chartStyleClone = Object.assign({},chartStyle);
  const seriesStyle = chartStyleClone.series;
  delete chartStyleClone.series;  
  
  if (seriesStyle){
    const chartSeries = chart.series || [];
    const applySeriesStyle = (series,index)=>{    
      LOGGER.info('before style apply', JSON.stringify(series));
      Object.assign(series,seriesStyle[index]);
      LOGGER.info('after style apply', JSON.stringify(series));
    }
    chartSeries.forEach(applySeriesStyle)  
  }  
  const finalChart = util.deepMergeJson(chart,chartStyleClone);
  return finalChart;
}

const columnVectorFromMatrix = (params)=>{
  const twoDimensionalData = params.data; 
  const columnIndex = params.columnIndex;
  
  const columnVector = [];
  
  const rowCount = twoDimensionalData.length;
  const colCount = twoDimensionalData[0].length;
  
  if (columnIndex > (colCount-1) ){
    throw 'column is not available in the given 2D data';
  }
  
  for (let rowIndex = 0;rowIndex < rowCount;rowIndex++){
    columnVector.push(twoDimensionalData[rowIndex][columnIndex]);
  }
  
  return columnVector;
}

const columnVectorAsSeries = (params)=>{
  const columnVector = params.data; 
  const name = columnVector[0];
  const data = columnVector.slice(1);  
  const series = {name,data}
  
  return series;
}


const mergePieChartDataAndStyle = (chart,chartStyle)=>{ 
  if (chartStyle.chart.type !== "pie"){
    return;
  } 
  const chartStyleClone = Object.assign({},chartStyle);  
  const seriesStyle = chartStyleClone.series;
  delete chartStyleClone.series;    
  
  if (seriesStyle){
    const chartSeries = chart.series || [];
    chartStyleClone.title.text = chartSeries[0].data[chartSeries[0].data.length-1];
    const applySeriesStyle = (series,index)=>{    
      LOGGER.info('before style apply for pie chart ', JSON.stringify(series));
      const donutTitle = series.data[series.data.length-1];
      series.data = series.data.slice(0,series.data.length-1);
      const currentSeriesStyle = seriesStyle[index];      
      const pointBuilder = (pointY,pointIndex)=>{
        const point = currentSeriesStyle.data[pointIndex] || {};
        point.name = chart.xAxis.categories[pointIndex];
        point.y = pointY;
        return point;
      };
      series.data = series.data.map(pointBuilder);            
      delete currentSeriesStyle.data;
      Object.assign(series,seriesStyle[index]);
      LOGGER.info('after style apply for pie chart ', JSON.stringify(series));
    }
    chartSeries.forEach(applySeriesStyle) 
  }  
  const finalChart = util.deepMergeJson(chart,chartStyleClone);
  return finalChart;
}

export {asHighchartModel}

if (require.main == module) {
  const params = {};
  params.data = [[null,"rd_ect_max_12","rd_ect_avg_12","rd_ect_max_13","rd_ect_avg_13"],[1507573800000,12.0,10.0,null,null],[1507660200000,6.0,5.0,null,null],[1507746600000,12.0,8.0,null,null],[1507833000000,14.0,12.0,null,null],[1507919400000,7.0,2.0,null,null],[1508005800000,12.0,4.0,null,null],[1508092200000,10.0,8.0,15.0,12.56],[1508178600000,null,null,12.0,15.0],[1508265000000,null,null,22.0,20.0]];
  
  asHighchartModel(params);
}