/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
/**
* Author : gneelagi
* Version : 1.0
*/

import path from 'path';
import fs from 'fs';
import * as util from '../util/util.js';
import * as chartUtil from './chart-util.js';
import env from 'os';
import React from 'react';
import ReactDOM from 'react-dom';
import ReactDOMServer from 'react-dom/server';
import { TitlePanelFactory, NotesPanel,FilterPanel } from '../table/components.js';
import { start as exporterStart, stop as exporterStop } from './chart-exporter.js';
import async from 'async';
import { ReportTitle } from '../table/simple-report-title.js';
import { WidgetTitle } from '../table/simple-widget-title.js';
import { KeyValueList } from '../table/simple-keyvalue-pair.js';
import { ValueList } from '../table/simple-valuelist.js';

const LOGGER = util.logger(__filename);

const chartdatagen = (props) => {
  const exporter = props.exporter,
    media = props.media, chartData = props.chartData,
    callback = props.callback;
  const widgetId = props.widgetKey;
  const program = util.program();
  const chartConfig = {};
  const chartStyle = props.chartStyle;
  const xAxisLabelDatetimeFormat = props.xAxisLabelDatetimeFormat;
  const measuresUomLocaleMessageId = props.measuresUomLocaleMessageId;
  LOGGER.info(' --props --', JSON.stringify(props));
  const config = chartData.config;
  let responseCount = 0;  
  const screenChartWidthHeight = (os = 'win32', span = 1) => {
    // it is intentionally repeated to make it flexible for screen,A4
    const chartWidthHeight = {};
    chartWidthHeight.height = 250;
    const leftRightSpacing = 4;
    // const leftRightBorder = 2;
    // const totalExtras = leftRightSpacing + leftRightBorder;
    const widgetsPerRow = 1;
    // const pageWidth = (program.width || 1200-(leftRightSpacing*widgetsPerRow));
    // const pageWidth = (program.width || 1200-(leftRightSpacing*widgetsPerRow));    
    // 72 DPI by linux - A4 = 8.27" x 11.69" = 595.44 * 841.68(596 * 842)
    // 96 DPI by windows  -A4 =  793.92 * 1122.24 (794 * 1123)    
    debugger;
    switch (os) {
      default:
      case 'win32': {
        const navigationMenuWidth = 1 * 96;
        const pageWidth = 1344 - (leftRightSpacing * widgetsPerRow) - navigationMenuWidth;
        const spanWidth = pageWidth / 3;
        chartWidthHeight.width = span * spanWidth;
        break;
      }
      case 'linux': {
        const navigationMenuWidth = 1 * 72;
        const pageWidth = 1008 - (leftRightSpacing * widgetsPerRow) - navigationMenuWidth;
        const spanWidth = pageWidth / 3;
        chartWidthHeight.width = span * spanWidth;
        break;
      }
    }

    return chartWidthHeight;
  }

  const a4ChartWidthHeight = (os = 'win32', span = 1) => {
    // it is intentionally repeated to make it flexible for screen,A4
    const chartWidthHeight = {};
    chartWidthHeight.height = 250;
    const leftRightSpacing = 4;
    // const leftRightBorder = 2;
    // const totalExtras = leftRightSpacing + leftRightBorder;
    const widgetsPerRow = 3;
    // const pageWidth = (program.width || 1200-(leftRightSpacing*widgetsPerRow));
    // const pageWidth = (program.width || 1200-(leftRightSpacing*widgetsPerRow));    
    // 72 DPI by linux - A4 = 8.27" x 11.69" = 595.44 * 841.68(596 * 842)
    // 96 DPI by windows  -A4 =  793.92 * 1122.24 (794 * 1123)   
    switch (os) {
      default:
      case 'win32': {
        const pageWidth = 794 - (leftRightSpacing * widgetsPerRow);
        const spanWidth = pageWidth / 3;
        chartWidthHeight.width = span * spanWidth;
        break;
      }
      case 'linux': {
        const pageWidth = 794 - (leftRightSpacing * widgetsPerRow);
        const spanWidth = pageWidth / 3;
        chartWidthHeight.width = span * spanWidth;
        break;
      }
    }

    return chartWidthHeight;
  }

  // const platform = env.platform();  
  const styleplatform = 'win32'; // 
  // let us use the same style across platform
  // else the style tailored for linux server accessed in non-linux machine 
  // will be having different look & feel(due to the DPI difference in windows,linux)
  const chartGenerator = (chartData, asyncCallback) => { 
    /*
    const chartConfig = JSON.parse(chartData.config.chart);
    chartConfig.series = chartData.series;
    chartConfig.series = chartConfig.series.map((singleSeries) => {
      Object.assign(singleSeries, singleSeries.config);
      return singleSeries
    });
    */
    chartData.config.chartStyle = chartStyle;
    chartData.config.xAxisLabelDatetimeFormat = xAxisLabelDatetimeFormat;
    chartData.config.measuresUomLocaleMessageId = measuresUomLocaleMessageId;
    const chartConfig = chartUtil.asHighchartModel(chartData);
    LOGGER.info('high charts config', JSON.stringify(chartConfig));
    // chartConfig.chart = {}; // for height & width  
	
    let span = 1;
    span = chartData.config.span || 3;
    let chartWidthHeight, outFile;
    switch (media) {
      default:
      case 'screen': {
        chartWidthHeight = screenChartWidthHeight(styleplatform, span); 
        break;
      }
      case 'A4': {
        chartWidthHeight = a4ChartWidthHeight(styleplatform, span);
        break;
      }
    }

    chartConfig.chart.height = chartWidthHeight.height;
    chartConfig.chart.width = chartWidthHeight.width;

    util.datasetInfo.chartInfo.series = chartConfig.series.length;
    util.datasetInfo.chartInfo.points = chartConfig.series.length ? chartConfig.series[0].data.length : 0;

    //Perform an export
    let time = new Date();
    time = time.getTime();
    outFile = path.join(props.reportdir, props.widgetKey + '.html');
    
    if (!chartConfig.hasSeries){
      //TODO extract common code into a method
      const headerprops = {};                   
      headerprops.note = [];
      
      headerprops.headerType = 'widget';
      headerprops.keyValueList = config.keyValueList || [] ;
      headerprops.title = config.title || {};
      headerprops.subTitle = config.subTitle || {};
      headerprops.valueList = config.valueList || [];
      
      
      const Header = (props) => {      
        return <section>
          <WidgetTitle title={props.title} subTitle = {props.subTitle}/>
          <ValueList valueList={props.valueList} />
          <NotesPanel note={props.note} />
        </section>;
      }
      const titleHtml = util.toHtml(Header,headerprops);
      const NoDataElement = (props)=>{
        return <section className="no-data-chart">{props.message} </section>
      }
      const noDataMessage = util.messages()['table.empty'];
      const noDataHtml = util.toHtml(NoDataElement,{message:noDataMessage})
      const tempHtml = "" + titleHtml + noDataHtml ;          
      const html = "<section class='widget'>" + tempHtml + "</section>";
      util.saveAsFile(outFile, html);
      asyncCallback(null, outFile);
      return;      
    }
    
    // const finalChartConfig = util.deepMergeJson(chartConfig,chartStyle);
    const chart = { "type": "svg", "outfile": outFile, "options": chartConfig };    
    exporter.export(chart, function (err, res) {
      // LOGGER.info(outFile);
      //The export result is now in res.
      //If the output is not PDF or SVG, it will be base64 encoded (res.data).
      //If the output is a PDF or SVG, it will contain a filename (res.filename).      
      // const svgFileName = res.filename;
      // ToDo make this reading file as a seperate task as IO is aSync.
      if (res) {
        const svgFileName = res.filename;
        try {
          const headerprops = {};                   
          headerprops.note = [];
          
          headerprops.headerType = 'widget';
          headerprops.keyValueList = config.keyValueList || [] ;
          headerprops.title = config.title || {};
          headerprops.subTitle = config.subTitle || {};
          headerprops.valueList = config.valueList || [];
          
          
          const Header = (props) => {      
            return <section>
              <WidgetTitle title={props.title} subTitle = {props.subTitle}/>
              <ValueList valueList={props.valueList} />
              <NotesPanel note={props.note} />
            </section>;
          }
          const titleHtml = util.toHtml(Header,headerprops);
          const tempHtml = "" + titleHtml + fs.readFileSync(svgFileName).toString() + "";          
          const html = "<section class='widget chart-container'>" + tempHtml + "</section>";
          util.saveAsFile(outFile, html);
        } catch (ex) {
          //TODO set proper level & control log based on level
          LOGGER.info('::: Exception :::', ex.stack, svgFileName, res.filename);
        }
      } else {
        //TODO set proper level & control log based on level
        LOGGER.info('::: Error from phantomjs svg conversion :::', res,err);
      } 

      
      asyncCallback(null, outFile);
    });

  }
  const asyncResult = (err, result) => {
    // util.memoryInfo.phantomjsChart +=  util.memoryInfo.phantomjsChartWorkerMem;  
    // TODO    
    const headerProps = {reportdir:props.reportdir};
    headerProps.widgetKey = props.widgetKey;  
    headerProps.filters = [];
    headerProps.title =  [];
    headerProps.note = [];     
    headerProps.callback = (err, headerHtml) => {
      result.splice(0, 0, headerHtml);
      util.joinFiles(result, result[result.length - 1], (err) => {        
        callback(err, result[result.length - 1]);
      });
    }
    util.saveHeaderAsHtml(headerProps);    
  }
  // mapSeries also should work
  // FIXME: we need to handle this via a hosted server
  const chartDataList = [];
  chartDataList.push(chartData);
  // async.concatSeries(chartDataList,chartGenerator,asyncResult);
  // chartGenerator(chartData);
  async.concatSeries(chartDataList, chartGenerator, asyncResult);
  // check for the chartWidgtes count if it is below or equals 8, a configurable item
  // run it else run it and take a callback, the call back should also do the same.  
  // create a function  that calls chart give the same function as the call back.        
}
const generator = { chartdatagen };
export default generator;

if (require.main === module) { // test the unit
  const chartData = { "config": { "recordsCount": 6, "recordsLimit": 4, "name": "collectionId=1", "chart": "{\"xAxis\":{\"type\":\"datetime\"}}" }, "series": [{ "name": "1", "data": [[1508047475762, 2.567], [1508047775762, 2.567], [1508048075762, 2.567]], "groups": ["1", "line"] }, { "name": "1", "data": [[1508047475762, 2.345], [1508047775762, 2.345], [1508048075762, 2.345]], "groups": ["1", "line"] }, { "name": "9", "data": [[1508047475762, 10.567], [1508047775762, 10.567], [1508048075762, 10.567]], "groups": ["2", "line"] }, { "name": "9", "data": [[1508047475762, 10.345], [1508047775762, 10.345], [1508048075762, 10.345]], "groups": ["2", "line"] }] };
  const callback = (err, result) => {
    console.log('completed', result);
  }
  const exporter = exporterStart();
  const widgetKey = '/home/govindan/report/502;1;0';
  const props = { exporter: exporter, media: 'screen', chartData: chartData, callback, widgetKey: widgetKey };
  const html = chartdatagen(props);
  console.log('::: HTML :::', html);
}