/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
/**
* Author : gneelagi
* Version : 1.0
*/

import generateChart from './simple-chart-enhanced.js';
import async from 'async';
import { start as exporterStart, stop as exporterStop } from './chart-exporter.js';
import path from 'path';
import * as util from '../util/util.js';

const LOGGER = util.logger(__filename);

const chartgen = (props) => {
  const chartDataHolder = props.dataHolder, callback = props.callback;
  const parentWidgetKey = props.parentWidgetKey;
  const exporter = exporterStart();
  const type = chartDataHolder.type;
  const config = chartDataHolder.config;
  const data = chartDataHolder.data;

  const style = config.style || 'widget-holder';
  const widgetKey = chartDataHolder.widgetId + ';' + chartDataHolder.instanceId;
  if (type !== 'chart') {
    //TODO set proper level & control log based on level
    LOGGER.info('only chart type supported by simple-chart');
    return null;
  }
  let index = 0;
  const generateChartForEachData = (chartData, callback) => {
    const widgetId = (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + chartDataHolder.widgetId + ";" + chartDataHolder.instanceId + ';' + index++;
    const widgetKey = widgetId; 
    const chartprops = { exporter: exporter, media: 'screen', chartData: chartData, callback: callback, widgetKey: widgetKey };
    chartprops.reportdir = props.reportdir;
    chartprops.chartStyle = util.styleJson(path.join(util.homeDir,'/conf/theme/default/chart'),config.chartStyle);
    chartprops.xAxisLabelDatetimeFormat = config.xAxisLabelDatetimeFormat;
    chartprops.measuresUomLocaleMessageId = config.measuresUomLocaleMessageId;
    // LOGGER.info( ' label ', chartprops.xAxisLabelDatetimeFormat );
    chartprops.title = chartData.config.title;
    
    chartprops.headerType = 'widget';
    chartprops.keyValueList = chartData.config.keyValueList || [] ;
    chartprops.title = chartData.config.title || {};
    chartprops.subTitle = chartData.config.subTitle || {};
    chartprops.valueList = chartData.config.valueList || [];
    
    
    generateChart.chartdatagen(chartprops);
  }
  const asyncResult = (err, result) => {
    // util.memoryInfo.phantomjsChart +=  util.memoryInfo.phantomjsChartWorkerMem;  
    // TODO enable utils 
    const destHtml = path.join(props.reportdir, (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetKey + '.html');

    const headerprops = {reportdir: props.reportdir};
    headerprops.headerType = 'widget';
    headerprops.keyValueList = config.keyValueList || [] ;
    headerprops.title = config.title || {};
    headerprops.subTitle = config.subTitle || {};
    //headerprops.valueList = config.valueList || [];
    headerprops.note = [];
    
    headerprops.widgetKey = (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetKey;
    headerprops.callback = (err, headerHtml) => {
      result.splice(0, 0, headerHtml);
      util.joinFiles(result, destHtml, (err) => {
        util.wrapHtmlWithStyle `<section class='${style}'>${destHtml}</section>`;
        callback(null, destHtml);
        // exporterStop(); 
        // TODO when we need to restart the highcharts exporter, will it automatically handled by the highcharts exporter if we don't kill the workers?
        // any limitations like only it will server 50, 0r 60 requests. Need to confirm the behavior.
      });
    }
    util.saveHeaderAsHtmlNew(headerprops);
  }
  // mapSeries also should work
  async.concatSeries(data, generateChartForEachData, asyncResult);
  // check for the chartWidgtes count if it is below or equals 8, a configurable item
  // run it else run it and take a callback, the call back should also do the same.  
  // create a function  that calls chart give the same function as the call back.      
  // return chartSvg;  
}

export { chartgen };

if (require.main === module) { // test the unit
 //  const dataHolder = { "type": "chart", "config": { "widgetId": "501", "widgetInstanceId": "4" }, "data": [{ "config": { "recordsCount": 6, "recordsLimit": 4, "name": "collectionId=1", "chart": "{\"xAxis\":{\"type\":\"datetime\"}}" }, "series": [{ "name": "1", "data": [[1508047475762, 2.567], [1508047775762, 2.567], [1508048075762, 2.567]], "groups": ["1", "line"] }, { "name": "1", "data": [[1508047475762, 2.345], [1508047775762, 2.345], [1508048075762, 2.345]], "groups": ["1", "line"] }, { "name": "9", "data": [[1508047475762, 10.567], [1508047775762, 10.567], [1508048075762, 10.567]], "groups": ["2", "line"] }, { "name": "9", "data": [[1508047475762, 10.345], [1508047775762, 10.345], [1508048075762, 10.345]], "groups": ["2", "line"] }] }, { "config": { "recordsCount": 4, "recordsLimit": 4, "name": "collectionId=2", "chart": "{\"xAxis\":{\"type\":\"datetime\"}}" }, "series": [{ "name": "1", "data": [[1508047475762, 2.567], [1508047775762, 2.567], [1508048075762, 2.567]], "groups": ["1", "line"] }, { "name": "1", "data": [[1508047475762, 2.345], [1508047775762, 2.345], [1508048075762, 2.345]], "groups": ["1", "line"] }, { "name": "9", "data": [[1508047475762, 10.567], [1508047775762, 10.567], [1508048075762, 10.567]], "groups": ["2", "line"] }, { "name": "9", "data": [[1508047475762, 10.345], [1508047775762, 10.345], [1508048075762, 10.345]], "groups": ["2", "line"] }] }] };
  const dataHolder = {"__type__":"null","type":"chart","widgetId":501,"instanceId":1,"config":{"widgetName":"Fabric","fetchTime":7142},"result":[{"series":[],"data":[[null,"rd_ect_max_12","rd_ect_avg_12","rd_ect_max_13","rd_ect_avg_13"],[1507573800000,12.0,10.0,null,null],[1507660200000,6.0,5.0,null,null],[1507746600000,12.0,8.0,null,null],[1507833000000,14.0,12.0,null,null],[1507919400000,7.0,2.0,null,null],[1508005800000,12.0,4.0,null,null],[1508092200000,10.0,8.0,15.0,12.56],[1508178600000,null,null,12.0,15.0],[1508265000000,null,null,22.0,20.0]],"config":{}}]}
  const callback = (err, result) => {
    console.log(' result', result);
  }
  const parentWidgetKey = '503;0';
  const props = { dataHolder: dataHolder, callback: callback, parentWidgetKey: parentWidgetKey };
  const html = chartgen(props);
}