/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
/**
* Author : gneelagi
* Version : 1.0
*/
import React from 'react';
import ReactDOMServer from 'react-dom/server';
import { compositedatagen } from './simple-composite.js';
import * as util from '../util/util.js';
import async from 'async';
import path from 'path';

const LOGGER = util.logger(__filename);

function compositegen(props) {
  /* A view for the data model for table in Spectre */
  const type = props.dataHolder.type;
  const config = props.dataHolder.config;
  const dataHolder = props.dataHolder;
  const data = props.dataHolder.data;
  const parentWidgetKey = props.parentWidgetKey;
  const callback = props.callback;
  let widgetKey
  if (dataHolder.widgetId) {
    widgetKey = dataHolder.widgetId + ';' + dataHolder.instanceId;
  } else {
    widgetKey = null;
  }
  if (!(type == 'composite' || type == 'report')) {
    //TODO set proper level & control log based on level
    LOGGER.info('only composite or report type supported by simple-composite');
    return null;
  }
  let index = 0;
  const widgetDataHolderFn = (widgetDataHolder, callback) => {
    let widgetKey;
    if (dataHolder.widgetId){
      widgetKey = (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + dataHolder.widgetId + ';' + dataHolder.instanceId + ';' + index++; 
    }else{
      widgetKey = (parentWidgetKey == '' ? '' : parentWidgetKey + ';')+index++;
    }   
    const compositeprops = { dataHolder: widgetDataHolder, callback: callback, widgetKey: widgetKey };
    compositeprops.reportdir = props.reportdir;
    compositeprops.compositeType = type; // report or composite
    compositeprops.layout = config.layout
    //Passing the composite widget holder style
    compositeprops.style = config.style;
    compositedatagen(compositeprops);
  }
  async.concatSeries(data, widgetDataHolderFn, (err, result) => {        
    let destHtml;    
    const headerprops = {reportdir:props.reportdir};
    headerprops.headerType = type || 'composite';
    headerprops.keyValueList = config.keyValueList || [] ;
    headerprops.title = config.title || {};
    headerprops.subTitle = config.subTitle || {};
    //headerprops.valueList = config.valueList || [];
    headerprops.note = [];
    
    if (widgetKey) {
      destHtml = path.join(headerprops.reportdir, (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetKey + '.html');
      headerprops.widgetKey = (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetKey;
    } else {
      destHtml = path.join(headerprops.reportdir, parentWidgetKey + '.html');
      headerprops.widgetKey = parentWidgetKey;
    }
    headerprops.callback = (err, headerHtml) => {      
      result.splice(0,0,headerHtml);
      util.joinFiles(result, destHtml, (err) => {
        util.wrapHtml `<section class='widget-holder'>${destHtml}</section>`;
        callback(null, destHtml);        
      });
    }        
    util.saveHeaderAsHtmlNew(headerprops);    
  });  
}

if (require.main === module) { // test the unit

  // const dataHolder = {"__type__":"null","type":"composite","result":[{"widgetsData":[{"__type__":"null","type":"table","widgetId":503,"instanceId":3,"config":{"widgetName":"Fabric"},"result":[{"data":[["ID","SRC_FCID","DST_FCID","RD_ECT_AVG"],[13,"SRC-002","DST-002",20.0],[13,"SRC-002","DST-002",15.0],[13,"SRC-002","DST-002",12.56],[12,"SRC-001","DST-001",12.0],[12,"SRC-001","DST-001",10.0]],"config":{},"columnDefinition":[]}]},{"__type__":"null","type":"chart","widgetId":501,"instanceId":1,"config":{"widgetName":"Fabric"},"result":[{"series":[],"data":[[null,"rd_ect_max_SRC-001--DST-001","rd_ect_avg_SRC-001--DST-001","rd_ect_max_SRC-002--DST-002","rd_ect_avg_SRC-002--DST-002"],[1507573800000,12.0,10.0,null,null],[1507660200000,6.0,5.0,null,null],[1507746600000,12.0,8.0,null,null],[1507833000000,14.0,12.0,null,null],[1507919400000,7.0,2.0,null,null],[1508005800000,12.0,4.0,null,null],[1508092200000,10.0,8.0,15.0,12.56],[1508178600000,null,null,12.0,15.0],[1508265000000,null,null,22.0,20.0]],"config":{}}]},{"__type__":"null","type":"table","widgetId":502,"instanceId":2,"config":{"widgetName":"Fabric"},"result":[{"data":[["ID","SRC_FCID","DST_FCID","RD_ECT_AVG"],[13,"SRC-002","DST-002",15.0],[13,"SRC-002","DST-002",20.0],[13,"SRC-002","DST-002",12.56]],"config":{},"columnDefinition":[]}]}],"config":{}}],"widgetId":503,"instanceId":3,"config":{"widgetName":"Host","fetchTime":7165}};
  const dataHolder = {"__type__":"null","type":"report","result":[{"widgetsData":[{"__type__":"null","type":"table","widgetId":502,"instanceId":2,"config":{"widgetName":"Switch","fetchTime":18396},"result":[{"data":[["ID","TIMESTAMP","FLOWID","RD_ECT_AVG","RD_ECT_MAX","WR_ECT_AVG","WR_ECT_MAX","COLLECTIONID"],[1,1508047475762,1,2.345,2.567,2.967,2.999,1],[5,1508047475762,5,6.345,6.567,6.967,6.999,1],[9,1508047475762,9,10.345,10.567,10.967,10.999,1],[12,1508047775762,1,2.345,2.567,2.967,2.999,1],[16,1508047775762,5,6.345,6.567,6.967,6.999,1]],"config":{},"columnDefinition":[]}]},{"__type__":"null","type":"chart","widgetId":501,"instanceId":1,"config":{"widgetName":"Fabric","fetchTime":18402},"result":[{"series":[],"data":[[null,"rd_ect_max_12","rd_ect_avg_12","rd_ect_max_13","rd_ect_avg_13"],[1507573800000,12.0,10.0,null,null],[1507660200000,6.0,5.0,null,null],[1507746600000,12.0,8.0,null,null],[1507833000000,14.0,12.0,null,null],[1507919400000,7.0,2.0,null,null],[1508005800000,12.0,4.0,null,null],[1508092200000,10.0,8.0,15.0,12.56],[1508178600000,null,null,12.0,15.0],[1508265000000,null,null,22.0,20.0]],"config":{}}]},{"__type__":"null","type":"composite","result":[{"widgetsData":[{"__type__":"null","type":"table","widgetId":503,"instanceId":3,"config":{"widgetName":"Fabric"},"result":[{"data":[["ID","SRC_FCID","DST_FCID","RD_ECT_AVG"],[13,"SRC-002","DST-002",20.0],[13,"SRC-002","DST-002",15.0],[13,"SRC-002","DST-002",12.56],[12,"SRC-001","DST-001",12.0],[12,"SRC-001","DST-001",10.0]],"config":{},"columnDefinition":[]}]},{"__type__":"null","type":"chart","widgetId":501,"instanceId":1,"config":{"widgetName":"Fabric"},"result":[{"series":[],"data":[[null,"rd_ect_max_SRC-001--DST-001","rd_ect_avg_SRC-001--DST-001","rd_ect_max_SRC-002--DST-002","rd_ect_avg_SRC-002--DST-002"],[1507573800000,12.0,10.0,null,null],[1507660200000,6.0,5.0,null,null],[1507746600000,12.0,8.0,null,null],[1507833000000,14.0,12.0,null,null],[1507919400000,7.0,2.0,null,null],[1508005800000,12.0,4.0,null,null],[1508092200000,10.0,8.0,15.0,12.56],[1508178600000,null,null,12.0,15.0],[1508265000000,null,null,22.0,20.0]],"config":{}}]},{"__type__":"null","type":"table","widgetId":502,"instanceId":2,"config":{"widgetName":"Fabric"},"result":[{"data":[["ID","SRC_FCID","DST_FCID","RD_ECT_AVG"],[13,"SRC-002","DST-002",15.0],[13,"SRC-002","DST-002",20.0],[13,"SRC-002","DST-002",12.56]],"config":{},"columnDefinition":[]}]}],"config":{}}],"widgetId":503,"instanceId":3,"config":{"widgetName":"Host","fetchTime":18411}}],"config":{}}],"widgetId":0,"instanceId":0,"config":{"layout":["501;1","502;2","503;3"],"fetchTime":18606}}
  const callback = (err, result) => {
    console.log('result', result)
  }
  const parentWidgetKey = '1';
  const props = { dataHolder: dataHolder, callback, parentWidgetKey };
  compositegen(props);
}
export { compositegen };

