/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
/**
* Author : gneelagi
* Version : 1.0
*/
import React from 'react';
import * as util from '../util/util.js';
import { tablegen } from '../table/simple-table-holder-enhanced.js';
import { chartgen } from '../chart/simple-chart-holder-enhanced.js';
import { compositegen } from '../composite/simple-composite-holder.js';
import path from 'path';
import async from 'async';

const LOGGER = util.logger(__filename);

function compositedatagen(props) {
  const style = props.style || 'widget-group';
  const config = props.dataHolder.config;
  config.layout = config.layout || props.layout // default to parent container layout
  const data = props.dataHolder.widgetsData;
  const widgetKey = props.widgetKey;
  const callback = props.callback;
  const type = props.compositeType;
  const widgetDataFn = (widgetDataHolder, callback) => {
    // const widgetDataHolder = JSON.parse(widgetsObject[widgetId]);
    switch (widgetDataHolder.type) {
      case 'table': {
        const tableprops = { dataHolder: widgetDataHolder, callback, parentWidgetKey: widgetKey };
        tableprops.reportdir = props.reportdir;
        tablegen(tableprops);
        break;
      }
      case 'chart': {
        const chartprops = { dataHolder: widgetDataHolder, callback, parentWidgetKey: widgetKey };
        chartprops.reportdir = props.reportdir;
        chartgen(chartprops);
        break;
      }
      case 'composite': {
        const compositeprops = { dataHolder: widgetDataHolder, callback, parentWidgetKey: widgetKey };
        compositeprops.reportdir = props.reportdir;
        compositegen(compositeprops);
      }
      default: {
        return '';
      }
    }
  }
  // const data = Object.keys(widgetsObject);
  async.concatSeries(data, widgetDataFn, (err, result) => {
    const destHtml = path.join(props.reportdir, widgetKey + '.html');
    const widgetHtmlFiles = (widgetId) => { return path.join(props.reportdir, widgetKey + ';' + widgetId + '.html') };
    const widgetSequence = config.layout || data.map((holder)=> holder.widgetId+';'+holder.instanceId);
    LOGGER.info('widget sequence , config.layout',widgetSequence,config.layout);
    const layoutResult = widgetSequence.map(widgetHtmlFiles);
    // as keys in a JSON object's ordering is not quaranteed.    
    const headerprops = {reportdir:props.reportdir};
    headerprops.widgetKey = props.widgetKey;
    headerprops.headerType = type;
    headerprops.keyValueList = config.keyValueList || [] ;
    headerprops.title = config.title || {};
    headerprops.subTitle = config.subTitle || {};
    headerprops.valueList = config.valueList || [];
    headerprops.note = [];
    
    headerprops.callback = (err, headerHtml) => {
      layoutResult.splice(0, 0, headerHtml);
      util.joinFiles(layoutResult, layoutResult[layoutResult.length - 1], (err) => {
        const targetHtml = layoutResult[layoutResult.length - 1]
        util.wrapHtmlWithStyle `<section class='${style}'>${targetHtml}</section>`;
        callback(err, targetHtml);
      });
    }
    // util.saveHeaderAsHtml(headerProps);
    util.saveHeaderAsHtmlNew(headerprops);
  });
}

if (require.main === module) { // test the unit 
  const dataHolder = {};
  const callback = (err, result) => {
    console.log(' result', result);
  }
  const widgetKey = '1;0';
  const props = { dataHolder: dataHolder, callback: callback, widgetKey: widgetKey };
  const html = compositedatagen(props);
}
export { compositedatagen }

