/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
  /**
 * Author : gneelagi
 * Version : 1.0
 */
import path from 'path';
import fs from 'fs';
import phantomjs from 'phantomjs-prebuilt';
import {logger} from '../../util/util.js';

const LOGGER = logger(__filename);

const generate = (schema,tableWidgetHtml,chartWidgetHtml,program,callback,killPostRun = false)=>{
  const t1 = new Date().getTime();
  const layout = schema.template.layout;
  let finalHtml = '';
  const rowRenderer = (row)=>{
    const cols = row.cols;
    let rowHtml = '';
    const colRenderer = (col)=> {      
      const className = "col-span-" + col.span;
      const colWidget = tableWidgetHtml[col.widget] || chartWidgetHtml[col.widget];
      const colDiv = `<div class="${className}"> ${colWidget}</div>`;
      rowHtml = rowHtml + colDiv;
    }
    cols.forEach(colRenderer);
    const rowClass = "report-row";
    const finalRowHtml = `<div class="${rowClass}">${rowHtml}</div>`;
    finalHtml = finalHtml + finalRowHtml;
  }
  layout.forEach(rowRenderer);     
  let reportHtml = path.join(program.reportdir,'report-a4.html');   
  fs.writeFileSync(reportHtml,finalHtml);    
  const reportPdf = reportHtml.replace('-a4.html','-a4.pdf');
  reportHtml = reportHtml.replace(/\\/g,'/');
  reportHtml = 'file:///'+ reportHtml;
  // const rasterizeLoc = path.join(phantomjs.location,"phantom","examples","rasterize.js");
  // const rasterizeLoc = path.join(__dirname,"pdf-generator-a4.js");
  const rasterizeLoc = path.join(__dirname,"pdf-generator-a4.js");
  // LOGGER.info('::: PHANTOM PATH :::',rasterizeLoc,reportHtml,reportPdf);
  // let paperSize = 'A4';
  // paperSize = ''; // TODO make printer-friendly
  LOGGER.info(' ::: PHANTOM CMD :::',rasterizeLoc,reportHtml,reportPdf);
  const cmd = phantomjs.exec(rasterizeLoc,reportHtml,reportPdf);
  cmd.on('exit',(code)=>{
    const t2 = new Date().getTime();
    // LOGGER.info('::: TIME TO RENDER REPORT :::',(t2-t1)/1000);
    callback.onReportEnd();
    if (killPostRun){
      process.exit(code);
    }    
  });       
  return 0;
}

const generator = {generate};

export default generator;

