/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
 
import {compositegen} from './composite/simple-composite-holder.js';
import {pdfgen} from './screen/pdf-generator-screen-enhanced.js';
import fs from 'fs';
import fsExtra from 'fs-extra';
import * as util from './util/util.js';
import path from 'path';
import {visitor as filterAndTitleVisitor} from './visitor/title-and-filter-visitor.js';
import {visitor as chartVisitor} from './visitor/chart-visitor.js';
import url,{URLSearchParams} from 'url';

const args = util.init();
var http = require('http');
var querystring = require('querystring');
const socketWriteTimeout = util.timeouts().serverSocketWriteTimeout || 3600000;
var server = http.createServer().setTimeout(socketWriteTimeout).listen(args.port);
const LOGGER = util.logger(__filename);

server.on('request', function (req, res) {   
	const params = new URLSearchParams(url.parse(req.url).query);
	const reportLocation = params.get('reportLocation');
	/* preview has values as a string */
	let preview = params.get('viewAsHtmlForPreview');
  LOGGER.info('location : ',req.url);
  
	try{
		preview = JSON.parse(preview);
	}catch(e){
		preview = false;
	}
    if (req.method == 'POST') {
        var body='';
    }
    req.on('data', function (data) {      
        body = body+data;      
    });
    req.on('end', function () {    
      try { 
        // TODO use async-await for promise based defered execution
        // avoid callback's use
        //  LOGGER.info(' body.toString',body);
        const reportJson = path.join(reportLocation , 'report.json');
        const postedContent = util.readJsonFile(reportJson); // JSON.parse(body);        
        filterAndTitleVisitor.visit(postedContent);
        chartVisitor.visit(postedContent);
        // we are supplying the host in the visit method instead of getting it during construction time        
        debugger;        
        LOGGER.info('postedContent ',JSON.stringify(postedContent));
        const dataHolder = postedContent;
        const callback = (err, result) => {
          LOGGER.info('result', result);
          LOGGER.info('temporary workspace dir',props.reportdir);
          util.cleanupWorkspace(props.reportdir);
          props.reporthtml = path.join(props.reportdir , 'report.html');
          const conf = {themeDir: path.join(util.homeDir,'conf/theme'),'theme':'default'};    
          props.stylecss =  path.join(conf.themeDir , conf.theme,'style.css');
          util.makeHtmlDocument(props); 
          // TODO make PDF generation separate & send status as events          
          const pdfgenprops = {htmlLoc:props.reporthtml};
          const writeResponse = () => {
            res.writeHead(200, {'Content-Type': 'text/html'});
            const  htmlFile = fs.readFileSync(result);
            res.end(htmlFile);
          };
      
          if(preview) {
            writeResponse();
            fsExtra.removeSync(props.reportdir);
            LOGGER.info('temporary workspace cleaned');
          } else {
            pdfgen(pdfgenprops).then(()=>{
              util.copyReportOutputToLocation(props.reportdir,reportLocation);
              fsExtra.removeSync(props.reportdir);
              LOGGER.info('temporary workspace cleaned');
              res.writeHead(200, {'Content-Type': 'text/html'});
              const  htmlFile = "<div>Report generated successfully</div>";
              res.end(htmlFile);
              // writeResponse();
            });
          }		  
        }
        const parentWidgetKey = 'report'; // the final output root document html
        const props = { dataHolder: dataHolder, callback, parentWidgetKey };
        props.reportdir = util.workspaceLocation(props.dataHolder.config.title);
        util.reportConfigProvider().reportTimeZoneId = dataHolder.config.reportTimeZoneId;
        compositegen(props);
		
      }catch(e){
        //TODO set proper level & control log based on level
        LOGGER.info(e.stack);
        res.writeHead(500, {'Content-Type': 'text/html'});
        res.end('<div>Server Error</div>'); // use the error react component
      }        
    });
   
});
LOGGER.info('nodejs options ENV,execArgv' , process.env.REPORT_GENERATOR_NODE_OPTIONS,process.execArgv);
LOGGER.info('NODE_ENV' , process.env.NODE_ENV);
LOGGER.info('html/pdf generator started on ' + args.port);