/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
import  scss from 'node-sass';
import fs from 'fs';
import path from 'path';
import { sync as DataURI } from 'datauri';
import fsExtra from 'fs-extra';

const tableScssFileA4 =  path.join(__dirname,"header-details-footer-a4.scss");
const tableScssFileScreenWin32 =  path.join(__dirname,"header-details-footer-screen-win32.scss");
const tableScssFileScreenLinux =  path.join(__dirname,"header-details-footer-screen-linux.scss");

const tableCssFileNameA4 = 'header-details-footer-a4.css';
const tableCssFileNameScreenWindows = 'header-details-footer-screen-win32.css';
const tableCssFileNameScreenLinux = 'header-details-footer-screen-linux.css';
const tableCopyToCssFileNameScreen = 'header-details-footer-screen.css';


const tableCssFileA4 =  path.join(__dirname,tableCssFileNameA4);
const tableCssFileScreenWindows =  path.join(__dirname,tableCssFileNameScreenWindows);
const tableCssFileScreenLinux =  path.join(__dirname,tableCssFileNameScreenLinux);

const render = (media,platform)=>{      
  const dual400EotNormalFontFilePath = "font/Dual-400.eot"; // ToDo get it from properties file
  const dual400EotNormalFontFile = path.join(__dirname,dual400EotNormalFontFilePath);
  const dual400EotNormalFontDataUri = DataURI(dual400EotNormalFontFile);
  
  const dual400TtfNormalFontFilePath = "font/Dual-400.ttf"; // ToDo get it from properties file
  const dual400TtfNormalFontFile = path.join(__dirname,dual400TtfNormalFontFilePath);
  const dual400TtfNormalFontDataUri = DataURI(dual400TtfNormalFontFile);
  
  switch (media){
    default:
    case 'screen':{
      const renderCss = (tableScssFileScreen,tableCssFileScreen)=>{
          const config = {
          "file": tableScssFileScreen          
        };
        const result = scss.renderSync(config);
        const cssFile = tableCssFileScreen;  
        const cssContent = result.css.toString();      
        let cssWithDataUri = cssContent.replace(new RegExp(dual400EotNormalFontFilePath),dual400EotNormalFontDataUri);
        cssWithDataUri = cssWithDataUri.replace(new RegExp(dual400TtfNormalFontFilePath),dual400TtfNormalFontDataUri);
        // use data uri
        fs.writeFileSync(cssFile,cssWithDataUri); 
      }      
      if (platform === 'win32'){
        renderCss(tableScssFileScreenWin32,tableCssFileScreenWindows);
      }else if (platform === 'linux'){
        renderCss(tableScssFileScreenLinux,tableCssFileScreenLinux);  
      }            
      break;
    }
    case 'A4':{
      const config = {
        "file": tableScssFileA4            
      };
      const result = scss.renderSync(config);
      const cssFile = tableCssFileA4;  
      fs.writeFileSync(cssFile,result.css);   
      break;
    }
  }  
}

const copyTo = (media,outFilePath,platform)=>{
  switch(media){
    default:
    case 'screen':{
      const writeCssFile = (tableCssFileScreen,tableCssFileNameScreen)=>{
        const source =  fs.readFileSync(tableCssFileScreen); 
        const destination =  path.join(outFilePath,tableCssFileNameScreen);
        fs.writeFileSync(destination,source);
      }
      if (platform === 'win32'){
        writeCssFile(tableCssFileScreenWindows,tableCopyToCssFileNameScreen);
      }else if (platform === 'linux'){
        writeCssFile(tableCssFileScreenLinux,tableCopyToCssFileNameScreen);
      }
            
      break;
    }
    case 'A4':{
      const source =  fs.readFileSync(tableCssFileA4); 
      const destination =  path.join(outFilePath,tableCssFileNameA4);
      fs.writeFileSync(destination,source);
      break;
    }
  } 
}
const compileAll = ()=>{
  compile('win32');
  compile('linux');
  //const fontFolder = path.join(__dirname,'font');
  //fsExtra.removeSync(fontFolder);
}
const compile = (platform)=>{
  render('screen',platform); 
  render('A4',platform); 
}

const generate = (media,outFilePath,platform)=>{  
  // compile(platform);  
  copyTo(media,outFilePath,platform);  
}


const generator = {generate}
export {compileAll}
export default generator