/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
  /**
 * Author : gneelagi
 * Version : 1.0
 */
import React from 'react';
import * as util from '../util/util.js';
class DetailsTable extends  React.Component {
  
  render(){
    const header = this.props.header;  
    const schema = this.props.schema;  
    const details = this.props.detail;
    
    const tHeader = <HeaderRow header={header} schema={schema}/>;
    const tDetail = (detailData,index)=><DetailRow key={index} header={header} schema={schema} detail={detailData}/>;
    
    let detailsArray = [] ;
    if (details.constructor.name !== 'Array'){     
      detailsArray.push(details);
    } else {
      detailsArray = details;
    }    
    const totalRecords = this.props.totalRecords || detailsArray.length;
    // for consistency sake , the framework will always have a single limit for 
    // max series supported per chart,
    // same applicable for table as well, so the consumer has to make sure
    // that the maxSeiresSupportedPerChart,maxSeriesSupportedPerTable 
    // is sufficient for chart appear in 3 in 1 widget
    // and the Top N table's N value is less than or equal to
    // maxRowsSupportedPerTable
    
    const fromIndex = 0;
    const toIndex = util.limits().maxRowsSupportedPerTable;
    const detailsArrayLimited = detailsArray.slice(fromIndex,toIndex);
        
    util.datasetInfo.tableInfo.rows = detailsArrayLimited.length;
    util.datasetInfo.tableInfo.columns = header.length;
    
    const NoteRow = (props)=>{
      return <div className="note-row" colSpan={props.cols}>{props.note}</div>
    }
    
    const noteMessage = util.messages()['table.note'];
    const noteMessageResolved = noteMessage.replace("{maxRowsSupportedPerTable}",toIndex);
    const noteRow = <NoteRow note={noteMessageResolved} cols={header.length}/>
    const includeNotes = (totalRecords > toIndex)? noteRow:null;
    
    const NoDataRow = (props)=>{
      return <tr><td className="table-nodata" colSpan={props.cols}>{props.noDataMessage}</td></tr>
    }
    const noDataMessage = util.messages()['table.empty'];
    const noDataRow = <NoDataRow noDataMessage={noDataMessage} cols={header.length}/>
    const includeNoDataMessage = (detailsArray.length === fromIndex)? noDataRow:null;
    
    const tDetails = detailsArrayLimited.map(tDetail);
    
    return <div>{includeNotes}<table><thead>{tHeader}</thead><tbody>{tDetails}{includeNoDataMessage}</tbody></table></div>;
  }
}

class HeaderRow extends  React.Component {
  render(){
    const row = this.props.header;
    const schema = this.props.schema;
    const tCol = (col,index)=> <HeadCol key={index} col={schema[col].view.displayName || schema[col]}/>;
    const tRow = row.map(tCol);
    
    return <tr>{tRow}</tr>;
  }
}
class DetailRow extends  React.Component {
  render(){
    const header = this.props.header;
    const schema = this.props.schema;
    const detail = this.props.detail;
    const tCol = (col,index)=> <Col key={index} col={detail[col]}/>;
    const tRow = header.map(tCol);
    
    return <tr>{tRow}</tr>;
  }
}
class Col extends  React.Component {
  render(){
    const col = this.props.col;    
    const tCol = col;    
    return <td>{tCol}</td>;
  }
}
class HeadCol extends  React.Component {
  render(){
    const col = this.props.col;    
    const thCol = col;    
    return <th>{thCol}</th>;
  }
}


export {DetailsTable}

