/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
/**
* Author : gneelagi
* Version : 1.0
*/
import React from 'react';
import * as util from '../util/util.js';
import { TitlePanelFactory, FilterPanel, NotesPanel } from './components.js';
import { ReportTitle } from './simple-report-title.js';
import { WidgetTitle } from './simple-widget-title.js';
import { KeyValueList } from './simple-keyvalue-pair.js';
import { ValueList } from './simple-valuelist.js';

class SimpleTableEnhanced extends React.Component {

  render() {    
    
    const detailProp = this.props.detail;        
    const header = detailProp.data[0];  
    const config = detailProp.config;  
    config.csvOnly = this.props.csvOnly;
    const details = [] // array slice does not work on 2D  
    // FIXME find a better way for this
    for(let index=1;index<detailProp.data.length;index++){
      details.push(detailProp.data[index])
    }        
    const tHeader = <HeaderRow header={header} />;
    const tDetail = (detailData, index) => <DetailRow key={index} detail={detailData} />;

    let detailsArray = details || [];    
    const totalRecords = this.props.detail.recordCount;
    const visibleRecords = detailsArray.length;        
    // for consistency sake , the framework will always have a single limit for 
    // max series supported per chart,
    // same applicable for table as well, so the consumer has to make sure
    // that the maxSeiresSupportedPerChart,maxSeriesSupportedPerTable 
    // is sufficient for chart appear in 3 in 1 widget
    // and the Top N table's N value is less than or equal to
    // maxRowsSupportedPerTable

    const fromIndex = 0;
    const toIndex = util.limits().maxRowsSupportedPerTable;
    // const detailsArrayLimited = detailsArray.slice(fromIndex, toIndex);

    util.datasetInfo.tableInfo.rows = detailsArray.length;
    util.datasetInfo.tableInfo.columns = header.length;

    const NoteRow = (props) => {
      return <div className="note-row" colSpan={props.cols}>{props.note}</div>
    }
    const noteMessage = util.messages()['table.note'] || "";
    const noteMessageResolved = noteMessage.replace("{maxRowsSupportedPerTable}", toIndex);
    const noteRow = <NoteRow note={noteMessageResolved} cols={header.length} />
    const includeNotes = (totalRecords > toIndex) ? noteRow : null;

    const NoDataRow = (props) => {
      return <tr><td className="table-nodata" colSpan={props.cols}>{props.noDataMessage}</td></tr>
    }
    const CsvOnlyRow = (props) => {	  
      const messageRowsBuilder = (displayMessage,index)=> {
        return <div key={index}>{displayMessage}</div>;
      }
      const finalMessage = props.messages.map(messageRowsBuilder);
      return <tr><td className="table-nodata" colSpan={props.cols}>{finalMessage}</td></tr>
    }
    
    let noDataRow;
    if (config.csvOnly){	
      const tableEmptyMessages = util.messages()['table.empty.for.csvonly'] || [];	
      noDataRow = <CsvOnlyRow messages={tableEmptyMessages} cols={header.length} />;
    } else {
      const noDataMessage = util.messages()['table.empty'];
      noDataRow = <NoDataRow noDataMessage={noDataMessage} cols={header.length} />;
    }
    const includeNoDataMessage = (detailsArray.length === fromIndex) ? noDataRow : null;

    const tDetails = detailsArray.map(tDetail);    

    const headerprops = {};
    
    headerprops.headerType = 'widget';
    headerprops.keyValueList = config.keyValueList || [] ;
    headerprops.title = config.title || {};
    headerprops.subTitle = config.subTitle || {};
    headerprops.valueList = config.valueList || [];	
    // headerprops.note =  this.props.note || [{ text: ['Notes goes here'], className: 'table-note' }];    
    // TODO need to use i18n library to pluralize, resolving text , etc.
    if (config.csvOnly){
      headerprops.note = [];		
    } else {
      const itemsMessage = util.messages()['table.note.items'] || "";
      const limitedItemsMessage = util.messages()['table.note'] || "";
      let itemsOrNoteMessage = totalRecords > visibleRecords? limitedItemsMessage : itemsMessage;
      if (visibleRecords === 1){
        itemsOrNoteMessage = util.messages()['table.note.item'] || "";
      }
      const itemsOrNoteMessageResolved = itemsOrNoteMessage.replace("{totalRecords}", totalRecords).replace("{visibleRecords}",visibleRecords);
      headerprops.note =  [{ text: [itemsOrNoteMessageResolved], className: 'table-note' }];
    }
    
    const Header = (props) => {      
      return <section>
          <WidgetTitle title={props.title} subTitle = {props.subTitle}/>
          <ValueList valueList={props.valueList} />
          <NotesPanel note={props.note} />
        </section>;
    }

    return <section className="widget"> <div><Header {...headerprops} /><table><thead>{tHeader}</thead><tbody>{tDetails}{includeNoDataMessage}</tbody></table></div></section>;
  }
}

class HeaderRow extends React.Component {
  render() {
    const row = this.props.header;           
    const tCol = (col, index) => <HeadCol key={index} col={col} />;
    const tRow = row.map(tCol);

    return <tr>{tRow}</tr>;
  }
}
class DetailRow extends React.Component {
  render() {        
    const row = this.props.detail;
    const tCol = (col, index) => <Col key={index} col={col} />;
    const tRow = row.map(tCol);

    return <tr>{tRow}</tr>;
  }
}
class Col extends React.Component {
  render() {
    const col = this.props.col;
    let tCol = col;
    if (col == null || (typeof col === "string" && col.trim() === "")){
      tCol = "-";
    }    
    return <td>{tCol}</td>;
  }
}
class HeadCol extends React.Component {
  render() {
    const col = this.props.col;
    const thCol = col;
    return <th>{thCol}</th>;
  }
}


export { SimpleTableEnhanced }

