/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
/**
* Author : gneelagi
* Version : 1.0
*/
import React from 'react';
import ReactDOMServer from 'react-dom/server';
import { SimpleTableEnhanced, TitlePanelFactory } from './components.js';
import * as util from '../util/util.js';
import async from 'async';
import path from 'path';

const LOGGER = util.logger(__filename);

function tablegen(props) {
  //TODO use let {a,b}={} pattern;    
  const dataHolder = props.dataHolder;
  const callback = props.callback;
  const parentWidgetKey = props.parentWidgetKey;
  const type = dataHolder.type;
  const config = dataHolder.config;
  const data = dataHolder.data;

  const style = config.style || 'widget-holder';
  const widgetHolderKey = dataHolder.widgetId + ';' + dataHolder.instanceId;
  if (type !== 'table') {
    //TODO set proper level & control log based on level
    LOGGER.info('only table type supported by simple-table');
    return null;
  }
  let index = 0;
  const tDetail = (detail, callback) => {    
    const widgetKey = (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetHolderKey +';' + index++;       
    const tableprops = {
      key: index, 
      detail: detail,widgetKey: widgetKey, callback: callback,
      reportdir: props.reportdir
    };
        
    tableprops.tableStyle = config.tableStyle;
    
    tableprops.headerType = 'widget';
    tableprops.keyValueList = config.keyValueList || [] ;
    tableprops.csvOnly = config.csvOnly;

    tableprops.title = config.title || {};
    tableprops.subTitle = config.subTitle || {};
    tableprops.valueList = config.valueList || [];
    
    // only holder has widget name , individual table could have extra information like matching collection name    
    tableprops.note = [];
    util.saveAsHtml(SimpleTableEnhanced, tableprops);
  }
  async.concatSeries(data, tDetail, (err, result) => {
    const destHtml = path.join( props.reportdir, (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetHolderKey + '.html');
    const headerprops = {reportdir: props.reportdir};    
    headerprops.note = [];
    
    headerprops.headerType = 'widget';
    headerprops.keyValueList = config.keyValueList || [] ;
    headerprops.title = config.title || {};
    headerprops.subTitle = config.subTitle || {};
    //headerprops.valueList = config.valueList || [];
    
    
    headerprops.widgetKey = (parentWidgetKey == '' ? '' : parentWidgetKey + ';') + widgetHolderKey;
    headerprops.callback = (err, headerHtml) => {
      result.splice(0, 0, headerHtml);
      util.joinFiles(result, destHtml, (err) => {        
        util.wrapHtmlWithStyle `<section class='${style}'>${destHtml}</section>`;
        callback(null, destHtml);
      });
    }
    util.saveHeaderAsHtmlNew(headerprops);
  });
}
export { tablegen };

if (require.main === module) { // test the unit
  // const dataHolder = { "type": "table", "config": { "schema": { "TIMESTAMP": { "displayName": "Last Modified Time New", "shortName": "1", "format": "dd/MMM/yyyy", "type": "int" }, "WR_ECT_MAX": { "displayName": "Write Exchange Completion Time (Max)", "shortName": "2" }, "WR_ECT_AVG": { "displayName": "Write Exchange Completion Time (Avg)", "shortName": "3" }, "COLLECTIONID": { "displayName": "Collection Id", "shortName": "4" }, "RD_ECT_MAX": { "displayName": "Read Exchange Completion Time (Max)", "shortName": "5" }, "RD_ECT_AVG": { "displayName": "Read Exchange Completion Time (Avg)", "shortName": "6" } }, "widgetId": "502", "widgetInstanceId": "2" }, "data": [{ "config": { "recordsCount": 9, "name": "1" }, "rows": [{ "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 2.999, "WR_ECT_AVG": 2.967, "COLLECTIONID": 1, "ID": 1, "RD_ECT_MAX": 2.567, "RD_ECT_AVG": 2.345, "FLOWID": 1 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 6.999, "WR_ECT_AVG": 6.967, "COLLECTIONID": 1, "ID": 5, "RD_ECT_MAX": 6.567, "RD_ECT_AVG": 6.345, "FLOWID": 5 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 10.999, "WR_ECT_AVG": 10.967, "COLLECTIONID": 1, "ID": 9, "RD_ECT_MAX": 10.567, "RD_ECT_AVG": 10.345, "FLOWID": 9 }, { "TIMESTAMP": 1508047775762, "WR_ECT_MAX": 2.999, "WR_ECT_AVG": 2.967, "COLLECTIONID": 1, "ID": 12, "RD_ECT_MAX": 2.567, "RD_ECT_AVG": 2.345, "FLOWID": 1 }, { "TIMESTAMP": 1508047775762, "WR_ECT_MAX": 6.999, "WR_ECT_AVG": 6.967, "COLLECTIONID": 1, "ID": 16, "RD_ECT_MAX": 6.567, "RD_ECT_AVG": 6.345, "FLOWID": 5 }] }, { "config": { "recordsCount": 12, "name": "2" }, "rows": [{ "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 3.999, "WR_ECT_AVG": 3.967, "COLLECTIONID": 2, "ID": 2, "RD_ECT_MAX": 3.567, "RD_ECT_AVG": 3.345, "FLOWID": 3 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 5.999, "WR_ECT_AVG": 5.967, "COLLECTIONID": 2, "ID": 4, "RD_ECT_MAX": 5.567, "RD_ECT_AVG": 5.345, "FLOWID": 4 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 8.999, "WR_ECT_AVG": 8.967, "COLLECTIONID": 2, "ID": 7, "RD_ECT_MAX": 8.567, "RD_ECT_AVG": 8.345, "FLOWID": 7 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 12.999, "WR_ECT_AVG": 12.967, "COLLECTIONID": 2, "ID": 11, "RD_ECT_MAX": 12.567, "RD_ECT_AVG": 12.345, "FLOWID": 11 }, { "TIMESTAMP": 1508047775762, "WR_ECT_MAX": 3.999, "WR_ECT_AVG": 3.967, "COLLECTIONID": 2, "ID": 13, "RD_ECT_MAX": 3.567, "RD_ECT_AVG": 3.345, "FLOWID": 2 }] }, { "config": { "recordsCount": 12, "name": "3" }, "rows": [{ "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 4.999, "WR_ECT_AVG": 4.967, "COLLECTIONID": 3, "ID": 3, "RD_ECT_MAX": 4.567, "RD_ECT_AVG": 4.345, "FLOWID": 3 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 7.999, "WR_ECT_AVG": 7.967, "COLLECTIONID": 3, "ID": 6, "RD_ECT_MAX": 7.567, "RD_ECT_AVG": 7.345, "FLOWID": 6 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 9.999, "WR_ECT_AVG": 9.967, "COLLECTIONID": 3, "ID": 8, "RD_ECT_MAX": 9.567, "RD_ECT_AVG": 9.345, "FLOWID": 8 }, { "TIMESTAMP": 1508047475762, "WR_ECT_MAX": 11.999, "WR_ECT_AVG": 11.967, "COLLECTIONID": 3, "ID": 10, "RD_ECT_MAX": 11.567, "RD_ECT_AVG": 11.345, "FLOWID": 10 }, { "TIMESTAMP": 1508047775762, "WR_ECT_MAX": 4.999, "WR_ECT_AVG": 4.967, "COLLECTIONID": 3, "ID": 14, "RD_ECT_MAX": 4.567, "RD_ECT_AVG": 4.345, "FLOWID": 3 }] }] };
  const dataHolder = {"__type__":"null","type":"table","widgetId":502,"instanceId":2,"config":{"widgetName":"Switch","fetchTime":7140},"result":[{"data":[["ID","TIMESTAMP","FLOWID","RD_ECT_AVG","RD_ECT_MAX","WR_ECT_AVG","WR_ECT_MAX","COLLECTIONID"],[1,1508047475762,1,2.345,2.567,2.967,2.999,1],[5,1508047475762,5,6.345,6.567,6.967,6.999,1],[9,1508047475762,9,10.345,10.567,10.967,10.999,1],[12,1508047775762,1,2.345,2.567,2.967,2.999,1],[16,1508047775762,5,6.345,6.567,6.967,6.999,1]],"config":{},"columnDefinition":[]}]}
  const callback = (err, result) => {
    console.log('result', result)
  }
  const parentWidgetKey = '0;0';
  const props = { dataHolder, callback, parentWidgetKey };
  tablegen(props);
}

