/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
  /**
 * Author : gneelagi
 * Version : 1.0
 */
import React from 'react';
import ReactDOMServer from 'react-dom/server';
import * as util from '../util/util.js';
import {TitlePanel}  from './title-panel.js';

class TitlePanelEnhanced extends  React.Component {
  
  render(){ 
    
    const headers = this.props.title || [];
    const title = headers[0] || [];
    const subtitle = headers[1] || []; // assume you always has two headers 
    // better validate this assumption in a validator or have a default headers  
    const Title = (props) => {
      const makeTitle = (headerText,index)=> {
        const resolvedMessage=util.resolveMessage(headerText);
        return <span key={index}>{resolvedMessage}</span>;
      }
      const header = props.title.text || [];
      const titles = header.map(makeTitle)      
      return <div className={props.title.className}>{titles}</div>;
    }
    const Subtitle = (props) => { 
      const makeTitle = (headerText,index)=> {
        const resolvedMessage=util.resolveMessage(headerText);
        return <span key={index} className="subtitle">{resolvedMessage}</span>;
      }
      const header = props.subtitle.text || [];
      const titles = header.map(makeTitle);      
      return <div className={props.subtitle.className}>{titles}</div>;
    }
    const titleElement = <div className="title-row-enhanced">
      <Title title={title}/>              
      <Subtitle subtitle={subtitle}/>
      {this.props.children}
    </div>;
            
    return titleElement;
  }
}



const titleCouldOverlap =  (titleInput)=>{
    const headers = titleInput || [];
    const title = headers[0] || {};
    const subtitle = headers[1] || {}; 
        
    const sumTitleLength = (sum,currentTitle)=>{return sum + (currentTitle+'').length};
    const titles =  title.text || [];
    const titleLength = (titles.reduce(sumTitleLength,0)/2) * 24/14; // normalize the characters length
    // NOT an eligant solution but we could live with this for now.
    
    const subtitles = subtitle.text || [];    
    const subtitleLength = subtitles.reduce(sumTitleLength,0);
    if (subtitleLength == 0){
      return false;
    }
    if(titleLength == 0){
      return false;
    }
    const separatorLength = 4;
    const totalTitleLength = titleLength + separatorLength + subtitleLength;     
    return totalTitleLength > 85;
}

const TitlePanelFactory =  (title)=>{
  if (titleCouldOverlap(title)){
    return TitlePanelEnhanced;
  }    
  return TitlePanel;
}



if (require.main === module){ // test the unit
  const schema = {"template":{"page":{"headers":[
  {"text":["Report Title"],className:"report-title"},
  {"text":["Sub Title 1","Sub Title 2"],className:"report-scope"}
  ]}}}


  const schemaFilter = {"template":{"page":{ "headers": {"title":[
    {"header":["Report Title"],className:"report-title"},
    {"header":["Sub Title 1","Sub Title 2"],className:"report-scope"}
    ],"filters":[{"filter":[{"text":["Filter Name:Exchange Server Flows 1"],"className":"report-filter"},{"text":["Initiator:23c801","Initiator 2:23c801","Initiator 3:23c801","Initiator 4:23c801"],"className":"report-filter"},{"text":["Target:24c801",["Target 2:24c801","LUN:2"],"Target 3:24c801","Target 4:24c801"],"className":"report-filter"}]},{"filter":[{"text":["Filter Name:Exchange Server Flows 2"],"className":"report-filter"},{"text":["Initiator:23c801","Initiator 2:23c801","Initiator 3:23c801","Initiator 4:23c801"],"className":"report-filter"},{"text":["Target:24c801","Target 2:24c801","Target 3:24c801","Target 4:24c801"],"className":"report-filter"}]}]
    }}}};
  
  const schemaNote = {"template":{"page":{"headers":{"note":[{text:['Notes goes here'],className:'table-note'}]}}}}
  const headers= schema.template.page.headers;
  const Title = TitlePanelFactory(headers);
  const html = ReactDOMServer.renderToString(
          <Title title={headers}/>);
          console.log('::: HTML :::',html);
}
export {TitlePanelFactory}