/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright © 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
  /**
 * Author : gneelagi
 * Version : 1.0
 */
// read template.json

// read widgets & widget spec

// read the report-data.json

// create report-view.json

// create a widget.json and allocate a worker for widget.html

// reduce the widget.html as report.html convert it to pdf

// phase 1: receive template.json,report.json 
// convert them as template-v1.json and report-v1.json

// template.json to have a chart widget
// the adapter will make necessary changes in template-v1.json and report-v1.json

// the algorithm to convert is as below
// clone template.json as template-v1.json , similarly for report.json to report-v1.json
// look up template-v1.json for max-series attribute and get the widget-ids
// look up the widget in widget-ids in report-v1.json  and for each widget
// get the series length.divide the series into max-series that should give no.of widget children
// assign a unique id for each children and update the template with layout information and widget meta-data

import fs from 'fs';
import path from 'path';
import * as util from './util.js';
const readJsonFromFile = util.readJsonFile;


//const schemaFileNameInput = 'template-v0.json';  // todo make it config driven
//const datasetFileNameInput = 'report-v0.json';  // todo make it config driven
//const schemaFileNameOutput = 'template-v1.json';  // todo make it config driven
//const datasetFileNameOutput = 'report-v1.json';  // todo make it config driven



const generate = (schemaFileNameInput = 'template-v0.0.json',
  datasetFileNameInput = 'report-v0.0.json',
  schemaFileNameOutput = 'template-v1.0.json',
  datasetFileNameOutput = 'report-v1.0.json' )=>{ 
  const maxChartsSupportedPerWidget = util.limits().maxChartsSupportedPerWidget;
  const maxSeriesSupportedPerChart = util.limits().maxSeriesSupportedPerChart;
  const program = util.program();    
  const schema = readJsonFromFile(path.join(program.reportdir,schemaFileNameInput));  
  const dataset = readJsonFromFile(path.join(program.reportdir,datasetFileNameInput));    
  const widgetFilter = (widgetId)=> {  
    const widgetView = schema.template.widgets[widgetId].view;
    const widgetDataset = dataset.report.widgets[widgetId];
    if (widgetView.as === 'chart'){
      const chartConfig = widgetView.config;
      const defaultMaxSeriesCount = maxSeriesSupportedPerChart; // from config
      // now consider all chart are max series charts with maximum set as 10 (from a config file)      
      chartConfig['max-series'] = chartConfig['max-series'] || defaultMaxSeriesCount;
      const maxSeries = chartConfig['max-series'];      
      const childrenSupported = (typeof chartConfig.childrenSupported === 'undefined')? true : chartConfig.childrenSupported;      
      if (maxSeries && maxSeries > 0 && widgetDataset.series.length > maxSeries){
        if (childrenSupported){
          return true;
        } else {          
          widgetDataset.series = widgetDataset.series.slice(0,maxSeries); 
          // for consistency sake , the framework will always have a single limit for 
          // max series supported per chart,
          // same applicable for table as well, so the consumer has to make sure
          // that the maxSeiresSupportedPerChart,maxSeriesSupportedPerTable 
          // is sufficient for chart appear in 3 in 1 widget
          // and the Top N table's N value is less than or equal to
          // maxRowsSupportedPerTable
          
          return false;
        }        
      } else {  
        if (widgetDataset.series.length === 0){
          chartConfig.options.subtitle = {'text':'No data to display'};
        }
        return false;
      }
    }  
  }
  const maxSeriesCharts = Object.keys(schema.template.widgets).filter(widgetFilter); 
  
  /*
  childWidgetSchema.view.config.childrenSupported = childWidgetSchema.view.config.childrenSupported || true;
        if (!childWidgetSchema.view.config.childrenSupported){
          return;
        }
  */
  
  const insertChildWidgetIfRequired =  (maxSeriesChart)=>{   
    let childCount = 1;    
    const firstChild = 1;
    const maxSeriesCount = schema.template.widgets[maxSeriesChart].view.config['max-series'] ;    
    const insertWidget = (widgetSchema,widgetDataset)=>{
       const seriesCount = widgetDataset.series.length;       
       if (seriesCount > maxSeriesCount){        
        delete schema.template.widgets[maxSeriesChart];
        delete dataset.report.widgets[maxSeriesChart];
        const childWidgetSchema = JSON.parse(JSON.stringify(widgetSchema));
        const childWidgetDataset = JSON.parse(JSON.stringify(widgetDataset));
        schema.template.widgets[maxSeriesChart+".child."+childCount] = childWidgetSchema;
        childWidgetDataset.series = childWidgetDataset.series.slice(0,maxSeriesCount);
        dataset.report.widgets[maxSeriesChart+".child."+childCount]=childWidgetDataset;        
        if (childCount !== firstChild){
          const view = schema.template.widgets[maxSeriesChart+".child."+childCount].view;
          view.headers = {};          
          const config = view.config;          
          config.child = true;
        }
        if (childCount === maxChartsSupportedPerWidget){               
          const view = schema.template.widgets[maxSeriesChart+".child."+firstChild].view;
          view.headers = view.headers || {};
          const headers = view.headers;          
          headers.note = [            
            {"text":["{message(chart.note)}"],"className":"chart-note"}
          ];
          return;
        }
        childCount++;
        const parentWidgetSchema = JSON.parse(JSON.stringify(widgetSchema));
        let parentWidgetDataset = JSON.parse(JSON.stringify(widgetDataset));
        parentWidgetDataset.series = parentWidgetDataset.series.slice(maxSeriesCount);
        const totalSeriesCountActual = parentWidgetDataset.totalRecordsCount;        
        const totalSeriesCountCalculated = maxChartsSupportedPerWidget * maxSeriesSupportedPerChart;
        const hasMoreSeriesThanLimit = totalSeriesCountActual > totalSeriesCountCalculated;        
        if (parentWidgetDataset.series.length <= maxSeriesCount ){
          schema.template.widgets[maxSeriesChart+".child."+childCount] = parentWidgetSchema;          
          dataset.report.widgets[maxSeriesChart+".child."+childCount] = parentWidgetDataset;    
          
          if (hasMoreSeriesThanLimit){
            const view = schema.template.widgets[maxSeriesChart+".child."+firstChild].view;
            view.headers = view.headers || {};
            const headers = view.headers;          
            headers.note = [            
              {"text":["{message(chart.note)}"],"className":"chart-note"}
            ];
          }
          // make title empty          
          const view = schema.template.widgets[maxSeriesChart+".child."+childCount].view;
          view.headers = {};          
          const config = view.config;          
          config.child = true;        
          // make title empty                    
          
        }else {        
          insertWidget(parentWidgetSchema,parentWidgetDataset);    
        }
      }
    }
  insertWidget(schema.template.widgets[maxSeriesChart],
    dataset.report.widgets[maxSeriesChart]);
    
  const schemaLayout = schema.template.layout;
  const insertRowIfRequired = (row,rowIndex)=>{
    // now consider a chart takes full row
    const fullRow =  true;
    const colsCount = row.cols.length;
    let index = 0;
    for(; index < colsCount; index++){
     if (row.cols[index].widget === maxSeriesChart){
       break;
     }
    }
    if (index === colsCount){
      return;
    }
    if (!fullRow){
      const maxSeriesWidgetSpan = row.cols[index].span;
      const maxColIndex = row.cols.length-1;
      if (index === maxColIndex){      
        const maxSeriesWidgetPrevSiblingSpan = row.cols[index-1].span;
        const maxSeriesWidgetPrevSiblingNewSpan = maxSeriesWidgetPrevSiblingSpan + maxSeriesWidgetSpan;
        row.cols[index-1].span = maxSeriesWidgetPrevSiblingNewSpan;  
      } else{
        const maxSeriesWidgetNextSiblingSpan = row.cols[index+1].span;
        const maxSeriesWidgetNextSiblingNewSpan = maxSeriesWidgetNextSiblingSpan + maxSeriesWidgetSpan;
        row.cols[index+1].span = maxSeriesWidgetNextSiblingNewSpan; 
      }    
    }    
    row.cols.splice(index,1); 
    
    let newRow;
    let newRowIndex = rowIndex;
    for(let newWidgetIndex = 0; newWidgetIndex < childCount; newWidgetIndex++ ){      
      if (fullRow || (newWidgetIndex % 3 === 0)){
        newRow = {};         
        newRow.cols = [];
        schemaLayout.splice(newRowIndex,0,newRow);  
        newRowIndex++;
      }     
      const  widgetIndex = (newWidgetIndex+1);    
      let newCol;
      if (fullRow){  
        newCol = {"widget":maxSeriesChart+ ".child." + widgetIndex, "span":3};         
      }else{
        newCol = {"widget":maxSeriesChart+ ".child." + widgetIndex, "span":1};   
      }      
      const lastWidgetIndex = childCount-1;
      newRow.cols.push(newCol);
      if (newWidgetIndex === lastWidgetIndex){        
        const maxSpan = 3;  
        const lastRowColCount = newRow.cols.length;
        const applySpan = (col)=>{          
          if (fullRow){
            col.span = 3
          } else {
            col.span = maxSpan/lastRowColCount;
          }
        }
        newRow.cols.forEach(applySpan);
      }      
    }    
  }  
  schemaLayout.forEach(insertRowIfRequired);    
  }  
  maxSeriesCharts.forEach(insertChildWidgetIfRequired);
  fs.writeFileSync(path.join(program.reportdir,schemaFileNameOutput),JSON.stringify(schema));
  fs.writeFileSync(path.join(program.reportdir,datasetFileNameOutput),JSON.stringify(dataset));
  // in layout locate the index of col of maxSeries Widget, get the span , add the span to previous col
  // create a new row , and pack the widgets of span 1, stretch the last widget with full span, if you have more // widgets then create new row and continue the same
 }

const generator = {generate};
export default generator;







