/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
 
import * as util from '../util/util.js';

util.init();
const LOGGER = util.logger(__filename);
const visitwidgetholder = (dataholder)=>{        
  const type = dataholder.type;
  if (type !== 'chart'){
    return dataholder;
  }
  const holderConfig = dataholder.config;
  const result = dataholder.data;
  const   dataNew =  [];
  const visitwidget = (widget)=>{  
    // consume filter definition and convert as view specific config object
    // for now return a config object for each data holder
    // TODO we may need to have a JSON schema for report json and validate against the report
    const config = widget.config;
    const maxSeries = util.limits().maxSeriesSupportedPerChart; 
    LOGGER.info('max series ',maxSeries);
    const data = widget.data;    
    const seriesCount = data[0].length-1;    
    const pointsPerSeries = data.length;   
    
    if (seriesCount > maxSeries){
        // [x,y1,y2,y3,y4]        
      const chartsRequired = Math.ceil(seriesCount / maxSeries);
      let seriesStart=1;
      let seriesEnd=seriesStart + maxSeries;
      for(let chartIndex = 0 ;chartIndex < chartsRequired;chartIndex++ ){ 
        const widgetNew = {};
        widgetNew.config = config;        
        widgetNew.subTitle = widget.subTitle;
        widgetNew.recordCount = widget.recordCount;
        const chartNew = [];            
        for(let pointIndex = 0; pointIndex  < pointsPerSeries;pointIndex++  ){
            const xPoint = data[pointIndex][0];
            const seriesNew = data[pointIndex].slice(seriesStart,seriesEnd);
            seriesNew.splice(0,0,xPoint);
            chartNew.push(seriesNew);
        } 
        widgetNew.data = chartNew;
        seriesStart = seriesEnd;
        seriesEnd = seriesStart + maxSeries; 
        dataNew.push(widgetNew);        
        LOGGER.info(JSON.stringify(chartNew));
      }                           
    }else {
      dataNew.push(widget);        
    }
    
    // TODO we need to have separate visitors for widget level filter definitions like threshold,etc.    
    return widget;  
  }  
  result.forEach(visitwidget);  
  dataholder.data = dataNew;
  return dataholder;  
}

const visitcompositewidgetholder = (dataholder)=>{     
 // now the assumption is report could have composite widgets
//  but composite widgets inside another composite widget is NOT tested.
  const holderConfig = dataholder.config;
  const result = dataholder.data;
  
  const visitcompositewidget = (compositewidget)=>{      
    const widgetsData = compositewidget.widgetsData; 
    const config = compositewidget.config;
    widgetsData.forEach(visitwidgetholder);      
    return compositewidget;  
  }
  result.forEach(visitcompositewidget);  
  return dataholder;
}

const chartvisitor = (reportholder)=>{    
  const result = reportholder.data; 
  const holderConfig = reportholder.config;
  const visitreport = (report)=>{  
    // consume filter definition and convert as view specific config object
    // for now return a config object for each data holder
    // TODO we may need to have a JSON schema for report json and validate against the report       
    const widgetsData = report.widgetsData;
    const visitbywidgettype = (dataholder)=>{
      switch (dataholder.type){        
        case 'chart':{
          visitwidgetholder(dataholder);  
          break;
        }
        case 'composite':{
          visitcompositewidgetholder(dataholder);  
          break;
        }
        default :{
            // raise error
        }
      }
    }
    widgetsData.forEach(visitbywidgettype);     
    // TODO we need to have separate visitors for widget level filter definitions like threshold,etc.    
    return report;  
  }
  result.forEach(visitreport);   
  // TODO we need to have separate visitors for widget level filter definitions like threshold,etc.    
  return reportholder;  
}


const reportvisitor = {visit:chartvisitor};

const visitors = [];

const add = (visitor)=>{
  visitors.push(visitor);
}

add(reportvisitor);

const visit = (report)=> {
  const visitorhandler = (visitor)=>{
    visitor.visit(report);
  }
  visitors.forEach(visitorhandler);  
}
// we are supplying the report object to all visitors
const visitor = {visit};

export {visitor};