/*-
 * UNPUBLISHED SOURCE CODE
 *
 * Copyright 2017, Brocade Communications Systems, Incorporated.
 * ALL RIGHTS RESERVED.
 *
 * BROCADE HIGHLY CONFIDENTIAL TRADE SECRET INFORMATION
 *
 * Access or possession of this material grants you no right or license,
 * express, implied, statutory or otherwise, under any Brocade patent,
 * copyright, trade secret right or other intellectual property right.
 * Any such license must be contained in a written license agreement
 * signed by an authorized officer of Brocade.  Additionally, access or
 * possession of this material is limited to Brocade employees with a
 * need to know or named employees of Brocade vendors authorized by
 * Brocade in writing to access this material.
 *
 * Restricted Rights: Use, duplication, or disclosure by the U.S.
 * Government is subject to restrictions of FAR 52.227-19 and its
 * successors, or (c)(1)(ii) of the Rights in Technical Data and
 * Computer Software clause at DFAR 252.227-7013 and its successors.
 *
 * Brocade Communications Systems, Inc.,
 * 110 Holger Way, San Jose, CA 95134
 */
import moment from 'moment-timezone';

const visitwidgetholder = (dataholder) => {
  const holderConfig = dataholder.config;
  const result = dataholder.data;
  const valueList = holderConfig.valueList;
  const visitwidget = (widget) => {
    // consume filter definition and convert as view specific config object
    // for now return a config object for each data holder
    // TODO we may need to have a JSON schema for report json and validate against the report
    const config = widget.config;
    config.title =
      { "text": holderConfig.widgetName, className: "widget-title" };
    config.subTitle =
      { "text": widget.subTitle, className: "widget-subtitle" };
    //the valueList contains ordered measure, etc.,
    config.valueList = valueList;
    // TODO we need to have separate visitors for widget level filter definitions like threshold,etc.    
    return widget;
  }
  result.forEach(visitwidget);
  return dataholder;
}

const visitcompositewidgetholder = (dataholder) => {
  // now the assumption is report could have composite widgets
  //  but composite widgets inside another composite widget is NOT tested.
  const holderConfig = dataholder.config;
  const result = dataholder.data;

  const visitcompositewidget = (compositewidget) => {
    const widgetsData = compositewidget.widgetsData;
    const config = compositewidget.config;
    // first widget of the composite to have the title
    widgetsData[0].data[0].subTitle = compositewidget.subTitle;

    widgetsData.forEach(visitwidgetholder);
    return compositewidget;
  }
  result.forEach(visitcompositewidget);
  return dataholder;
}

const visitreportholder = (reportholder) => {
  const result = reportholder.data;
  const holderConfig = reportholder.config;
  const filters = holderConfig.filters;
  const networkScope = holderConfig.networkScope;
  const timeScope = holderConfig.timeScope;
  const granularity = holderConfig.granularity;
  const reportTimeZoneId = holderConfig.reportTimeZoneId;
  let delimiter = '-';
  const dateTimeformat = 'MMM D, YYYY, HH:mm z';
  const visitreport = (report, index) => {
    // consume filter definition and convert as view specific config object
    // for now return a config object for each data holder
    // TODO we may need to have a JSON schema for report json and validate against the report    
    const config = report.config;
    var startDate = moment.tz(timeScope[index].startDate, reportTimeZoneId).format(dateTimeformat);
    var endDate = moment.tz(timeScope[index].endDate, reportTimeZoneId).format(dateTimeformat);
    var timeScopeForReport = " " + startDate + " " + delimiter + " " + endDate;
    config.title =
      { "text": holderConfig.title, className: "report-title" };
    const filtersText = filters.map((filter)=>filter.name);    
    //Filter is pending add filter before NetworkScope in keyValueList
    config.keyValueList = [];
    if (filtersText && filtersText.length > 0){
      config.keyValueList.push( { "key": "Filters", "value": " " + filtersText.join(","), "className": null });  
    }    
    config.keyValueList.push( { "key": "Network Scope", "value": " " + networkScope, "className": null });
    config.keyValueList.push( { "key": "Date Range", "value": " " + timeScopeForReport, "className": null });
    // config.keyValueList.push( { "key": "Granularity", "value": " " + granularity, "className": null });        
	// Granularity is forcefully hidden as we don't want this in 1.1.0
	// When it is required to be shown later , just uncomment  this.

    const widgetsData = report.widgetsData;
    const visitbywidgettype = (dataholder) => {
      switch (dataholder.type) {
        case 'table':
        case 'chart': {
          visitwidgetholder(dataholder);
          break;
        }
        case 'composite': {
          visitcompositewidgetholder(dataholder);
          break;
        }
        default: {
          // raise error
        }
      }
    }
    widgetsData.forEach(visitbywidgettype);
    // TODO we need to have separate visitors for widget level filter definitions like threshold,etc.    
    return report;
  }
  result.forEach(visitreport);
  // TODO we need to have separate visitors for widget level filter definitions like threshold,etc.    
  return reportholder;
}


const reportvisitor = { visit: visitreportholder };

const visitors = [];

const add = (visitor) => {
  visitors.push(visitor);
}

add(reportvisitor);

const visit = (report) => {
  const visitorhandler = (visitor) => {
    visitor.visit(report);
  }
  visitors.forEach(visitorhandler);
}
// we are supplying the report object to all visitors
const visitor = { visit };

export { visitor };